/*-----------------------------------------------------------------------------
  Project:  QRE
  File:     qre.h

  Copyright (C)2010 Nintendo Co., Ltd.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.

  $Revision: 1.10 $
 *----------------------------------------------------------------------------*/
#ifndef MW_QRE_QRE_H_
#define MW_QRE_QRE_H_

//==============================================================================

#include <nn.h>

#ifdef __cplusplus

namespace mw {
namespace qre {

/* Please see man pages for details */
typedef enum EccLevel
{
    ECC_LEVEL_L = 0,
    ECC_LEVEL_M,
    ECC_LEVEL_Q,
    ECC_LEVEL_H
} EccLevel;

/* Please see man pages for details */
typedef struct EncodeData
{
    u32 size;           /* Please see man pages for details */
    u8* data;           /* Please see man pages for details */
    u32 version;        /* Please see man pages for details */
    u32 total;          /* Please see man pages for details */
    u32 cell_size;      /* Please see man pages for details */
    EccLevel ecc_level; /* Please see man pages for details */
    NN_PADDING3;
} EncodeData;

/* Please see man pages for details */
typedef struct Rect
{
    s32 top;            /* Please see man pages for details */
    s32 left;           /* Please see man pages for details */
    s32 right;          /* Please see man pages for details */
    s32 bottom;         /* Please see man pages for details */
} Rect;

/* Please see man pages for details */
typedef struct ImageInfo
{
    u16 width;          /* Please see man pages for details */
    u16 height;         /* Please see man pages for details */
    u8* rgb888Data;     /* Please see man pages for details */
    Rect imgPos;        /* Please see man pages for details */
} ImageInfo;

//==============================================================================

/* Please see man pages for details */
class QREncoder
{
public:
    /* Please see man pages for details */
    QREncoder();
    /* Please see man pages for details */
    virtual ~QREncoder();

    /* Please see man pages for details 
     
     
     
     
     
     */
    bool Encode(EncodeData* qrInfo);

    /* Please see man pages for details 
     
     
     
     
     
     
     */
    bool EncodeWithImage(EncodeData* qrInfo, const ImageInfo* imageInfo);

    /* Please see man pages for details 
     
     
     
     
     
     
     */
    bool GetErrorMessage(char* pMess, int size);

    /* Please see man pages for details 
     
     
     
     */
    u32 GetDivCount();

    /* Please see man pages for details 
     
     
     
     */
    u32 GetVersion();

    /* Please see man pages for details 
     
     
     
     
     
     */
    u32 GetQRSize(u32 index = 0, bool isBmp = false);

    /* Please see man pages for details 
     
     
     
     
     
     
     
     
     */
    bool GetQRBMPData(u8* buff, u32 size, u32 index = 0, mw::qre::Rect *pos = NULL);

    /* Please see man pages for details 
     
     
     
     
     
     
     */
    bool GetQRData(ImageInfo* info, u32 index = 0);

    /* Please see man pages for details 
     
     
     
     
     
     
     */
    bool InitMemory(void* ptr, u32 size);

    /* Please see man pages for details 
     
     
     
     
     
     
     */
    static u32 GetEncodeBufferSize(u32 data_size, u32 cell_size, u32 count = 0);

    /* Please see man pages for details 
     
     
     
     
     
     */
    static u32 GetDesignQRImageSize(u32 version, u32 cellSize);
};

} // namespace qre
} // namespcae mw

#endif	// __cplusplus

//==============================================================================

#ifdef __cplusplus
# define MW_QRE_EXTERN_C extern "C"
#else
# define MW_QRE_EXTERN_C extern
#endif

/* Please see man pages for details 
 
 
 
 */
typedef union mwqreQREncoder
{
	char buf[4];
	u32 alignment_holder;
} mwqreQREncoder;

/* Please see man pages for details */
typedef enum mwqreEccLevel
{
    MW_QRE_ECC_LEVEL_L = 0,
    MW_QRE_ECC_LEVEL_M,
    MW_QRE_ECC_LEVEL_Q,
    MW_QRE_ECC_LEVEL_H
} mwqreEccLevel;

/* Please see man pages for details */
typedef struct mwqreEncodeData
{
    u32 size;                /* Please see man pages for details */
    u8* data;                /* Please see man pages for details */
    u32 version;             /* Please see man pages for details */
    u32 total;               /* Please see man pages for details */
    u32 cell_size;           /* Please see man pages for details */
    mwqreEccLevel ecc_level; /* Please see man pages for details */
    NN_PADDING3;
} mwqreEncodeData;

/* Please see man pages for details */
typedef struct mwqreRect
{
    s32 top;                 /* Please see man pages for details */
    s32 left;                /* Please see man pages for details */
    s32 right;               /* Please see man pages for details */
    s32 bottom;              /* Please see man pages for details */
} mwqreRect;

/* Please see man pages for details */
typedef struct mwqreImageInfo
{
    u16 width;               /* Please see man pages for details */
    u16 height;              /* Please see man pages for details */
    u8* rgb888Data;          /* Please see man pages for details */
    mwqreRect imgPos;        /* Please see man pages for details */
} mwqreImageInfo;

/* Please see man pages for details 
 
 */
MW_QRE_EXTERN_C bool mwqreQREncoderEncode(mwqreQREncoder* this_, mwqreEncodeData* qrInfo);

/* Please see man pages for details 
 
 */
MW_QRE_EXTERN_C bool mwqreQREncoderEncodeWithImage(mwqreQREncoder* this_, mwqreEncodeData* qrInfo, const mwqreImageInfo* imageInfo);

/* Please see man pages for details 
 
 */
MW_QRE_EXTERN_C bool mwqreQREncoderGetErrorMessage(mwqreQREncoder* this_, char* pMess, int size);

/* Please see man pages for details 
 
 */
MW_QRE_EXTERN_C u32 mwqreQREncoderGetDivCount(mwqreQREncoder* this_);

/* Please see man pages for details 
 
 */
MW_QRE_EXTERN_C u32 mwqreQREncoderGetVersion(mwqreQREncoder* this_);

/* Please see man pages for details 
 
 */
MW_QRE_EXTERN_C u32 mwqreQREncoderGetQRSize(mwqreQREncoder* this_, u32 index, bool isBmp);

/* Please see man pages for details 
 
 */
MW_QRE_EXTERN_C bool mwqreQREncoderGetQRBMPData(mwqreQREncoder* this_, u8* buff, u32 size, u32 index, mwqreRect* pos);

/* Please see man pages for details 
 
 */
MW_QRE_EXTERN_C bool mwqreQREncoderGetQRData(mwqreQREncoder* this_, mwqreImageInfo* info, u32 index);

/* Please see man pages for details 
 
 */
MW_QRE_EXTERN_C bool mwqreQREncoderInitMemory(mwqreQREncoder* this_, void* ptr, u32 size);

/* Please see man pages for details 
 
 */
MW_QRE_EXTERN_C u32 mwqreQREncoderGetEncodeBufferSize(u32 data_size, u32 cell_size, u32 count);

/* Please see man pages for details 
 
 */
MW_QRE_EXTERN_C u32 mwqreQREncoderGetDesignQRImageSize(u32 version, u32 cellSize);

#endif // MW_QRE_QRE_H_
// EOF
